
// SPGGenome.cpp
// Copyright 2006 Steve Gargolinski

/*
This file is part of SPGGenomePool.

SPGGenomePool is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

SPGGenomePool is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SPGGenomePool; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include "SPGGenome.h"
#include <math.h>

//
// Constructor
//
SPGGenome::SPGGenome()
{
	m_numBits = 0;
	m_data.clear();
	m_strength = 0.0f;
}

//
// Constructor
//
SPGGenome::SPGGenome(int numBits)
{
	// Make sure that we don't make a chromosome with a bogus number of bits
	if (numBits > 0) 
	{
		m_numBits = numBits;
	} 
	else 
	{
		m_numBits = 0;
	}

	m_data.clear();
	m_strength = 0.0f;
}

//
// Constructor
//
SPGGenome::SPGGenome(const std::vector<int>& newChromo)
{
	// First reset the data and strength
	m_data.clear();
	m_strength = 0.0f;

	// Now see the number of bits and copy then over from the passed vector
	m_numBits = (int)newChromo.size();

	for (size_t i = 0; i < newChromo.size(); ++i) 
	{
		m_data.push_back(newChromo[i]);
	}
}

//
// Fills up this genome with a random series of 1s and 0s
//
void SPGGenome::FillGenomeBinary()
{
	// Make sure that we don't have a bogus length
	if (m_numBits > 0) 
	{
		// First clear out the data
		m_data.clear();

		// Now add n random 1s or 0s
		for (int i = 0; i < m_numBits; ++i) 
		{
			int val = rand() % 2;
			m_data.push_back(val);
		}
	}
}

//
// Takes in a float [0, 1] - For each bit in this chromosome, possibly
// flips the bit depending on the mutation rate.  Returns true if at least
// one bit in the chromosome was flipped, false otherwise
//
bool SPGGenome::Mutate(float mutationRate)
{
	bool retValue = false;

	// Loop through each bit of the chromosome
	for (size_t i = 0; i < m_data.size(); ++i)
	{	
		// Get a random float from [0, 1]
		float r = rand() / (RAND_MAX + 1.0f);

		// Check to see if this random value is less than the mutationRate passed in
		if (r <= mutationRate) 
		{
			// If so then flip this bit
			if (m_data[i] == 0) 
			{
				m_data[i] = 1;
			} 
			else 
			{
				m_data[i] = 0;
			}

			retValue = true;
		}
	}

	return retValue;
}