

// SPGGenome.h
// Copyright 2006 Steve Gargolinski

/*
This file is part of SPGGenomePool.

SPGGenomePool is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

SPGGenomePool is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SPGGenomePool; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef __SPGGENOME_H__
#define __SPGGENOME_H__

#include <vector>

//
// For now this is a random string of bits (1s and 0s for now) to be used in a genetic algrithm
//
// FillGenomeBinary() should be swapped out with a strategy pattern or something to facilitate
// things other than simple binary strings.
//

class SPGGenome
{
public:

	//
	// A few different constructors
	//
	SPGGenome();
	SPGGenome(int numBits);
	SPGGenome(const std::vector<int>& newChromo);

	
	//
	// Returns the number of bits in this genome
	//
	int GetNumBits() { return m_numBits; }

	//
	// Returns the strength
	//
	float GetStrength() { return m_strength; }

	//
	// Returns our bit string as a vector of ints
	//
	std::vector<int> GetData() { return m_data; }

	//
	// Sets the number of bits in this genome
	//
	void SetNumBits(int numBits) { m_numBits = numBits;	}

	//
	// Set the strength of this genome
	//
	void SetStrength(float newStrength) { m_strength = newStrength;	}

	//
	// Sets the data (binary string) of this genome
	//
	void SetData(const std::vector<int>& newData) { m_data = newData; }

	//
	// Fills up this genome with a random series of 1s and 0s
	//
	void FillGenomeBinary();

	//
	// Takes in a float [0, 1] - For each bit in this genome, possibly
	// flips the bit depending on the mutation rate.  Returns true if at least
	// one bit in the genome was flipped, false otherwise
	//
	bool Mutate(float mutationRate);

private:

	// The number of bits in this genome
	int m_numBits;

	// The internal representation of the bits
	std::vector<int> m_data;

	// The strength of this genome
	float m_strength;

};


#endif